@@ -29,7 +29,7 @@
 ## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 ## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ## 
-## $Id: $
+## $Id: dcemri_bayes.R 332 2010-01-29 16:54:07Z bjw34032 $
 ##
 #############################################################################
 ## setGeneric("dcemri.bayes")
@@ -165,7 +165,9 @@ setMethod("dcemri.bayes", signature(conc="array"),
 
   mod <- model
   nvoxels <- sum(img.mask)
-  I <- nrow(conc) ;  J <- ncol(conc) ;  K <- nsli(conc)
+  I <- nrow(conc)
+  J <- ncol(conc)
+  K <- nsli(conc)
   
   if (!is.numeric(dim(conc))) {
     I <- J <- K <- 1
@@ -178,6 +180,7 @@ setMethod("dcemri.bayes", signature(conc="array"),
   img.mask <- array(img.mask,c(I,J,K))
  
   if (verbose) cat("  Deconstructing data...", fill=TRUE)
+  img.mask <- ifelse(img.mask > 0, TRUE, FALSE)
   conc.mat <- matrix(conc[img.mask], nvoxels)
   conc.mat[is.na(conc.mat)] <- 0
 
